/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.drill.exec.ref.values;

import java.io.IOException;

import org.apache.drill.common.expression.PathSegment;
import org.apache.drill.common.types.TypeProtos.MajorType;
import org.apache.drill.exec.ref.rops.DataWriter;


public interface DataValue {
  
  public static final DataValue NULL_VALUE = new ScalarValues.NullValue();

  public DataValue getValue(PathSegment segment);
  public void addValue(PathSegment segment, DataValue v);
  public void removeValue(PathSegment segment);
  public void write(DataWriter writer) throws IOException;
  public MajorType getDataType();
  public NumericValue getAsNumeric();
  public ContainerValue getAsContainer();
  public StringValue getAsStringValue();
  public BooleanValue getAsBooleanValue();
  public BytesValue getAsBytesValue();
  public boolean equals(DataValue v);
  public boolean equals(Object v);
  public int hashCode();
  public DataValue copy();
}
